import argparse
import os
from urllib import request
from shutil import unpack_archive
import datetime
import subprocess
import json

def script_cli():
    parser = argparse.ArgumentParser()
    parser.add_argument("--file",
                        help="JSON options file",
                        type=str)
    parser.add_argument("--tool_data_table_name",
                        help="Tool data table name",
                        type=str)
    args = parser.parse_args()

    with open(args.file) as params_file:
        params = json.load(params_file)

    target_directory = params["output_data"][0]["extra_files_path"]
    os.mkdir(target_directory)

    taxid = params["param_dict"]["taxid"]
    node_name = params["param_dict"]["node_name"]
    name = " ".join([node_name, f"(taxid - {taxid})", f"(date - {datetime.datetime.now().strftime('%Y_%m_%d_%H')})"])
    tool_data_table_name = args.tool_data_table_name
    data_id = "_".join([taxid, datetime.datetime.now().strftime("%Y_%m_%d_%H_%M_%S")])

    url = 'ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz'
    request.urlretrieve(url, os.path.join(target_directory, "taxdump.tar.gz"))
    unpack_archive(os.path.join(target_directory, "taxdump.tar.gz"), extract_dir=target_directory)

    subprocess.call(
        f"taxonkit list --ids {taxid} --data-dir {target_directory}" 
        f"| taxonkit filter --equal-to Species --lower-than Species --save-predictable-norank --data-dir {target_directory}" 
        f"> {os.path.join(target_directory, data_id)}.txids",
        shell=True
    )

    data_table_entry = {
        "value": data_id,
        "name": name,
        "node_name": node_name,
        "path": os.path.join(taxid, data_id)+".txids"
    }
    data_manager_dict = {
        "data_tables": {tool_data_table_name: [data_table_entry]}
    }
    output_json = open(args.file, "w")
    output_json.write(json.dumps(data_manager_dict))
    output_json.close()


if __name__ == "__main__":
    script_cli()
